@@ -1,7 +1,7 @@
 package Git::LoadCPAN::Error;
-use 5.008;
+use 5.008001;
 use strict;
-use warnings;
+use warnings $ENV{GIT_PERL_FATAL_WARNINGS} ? qw(FATAL all) : ();
 use Git::LoadCPAN (
 	module => 'Error',
 	import => 1,
